
package jp.ac.takushoku_u.cs;
import java.util.Set;
import java.util.Iterator;
import java.util.TreeSet;

/**
* LFTAnalysisクラスはLFT分析を行うためのクラスです。<BR>
* また、このクラスを利用し隣接マトリクスにおける枝の重要度やグラフ間の類似度、差異度、対象を要素集合、有向枝集合、要素の順序対集合を求めることができます。
* @version 1.4.1
* @author 河原 慎治(Kawahara Shinji)
*/
/*
* 最終更新日 2005年11月07日
*/
public class LFTAnalysis{
	
/**
* 教授者のマトリクスを保持するマトリクス
*/
	private BooleanMatrix teacherMatrix;
	
/**
* 学習者のマトリクスを保持するマトリクス
*/
	private BooleanMatrix studentMatrix;
	
/**
* 有向枝がある場合を表す定数
*/
	private static final int LFT_EDGE_INT_TRUE = 1;
	
/**
* 有向枝がある場合を表す定数
*/
	private static final boolean LFT_EDGE_TRUE = true;
	
/**
* 有向枝がある場合を表す定数
*/
	private static final String LFT_EDGE_STRING_TRUE = "1";
	
/**
* 有向枝がない場合を表す定数
*/
	private static final int LFT_EDGE_INT_FALSE = 0;
	
/**
* 有向枝がない場合を表す定数
*/
	private static final boolean LFT_EDGE_FALSE = false;
	
/**
* 有向枝がない場合を表す定数
*/
	private static final String LFT_EDGE_STRING_FALSE = "0";
	
/**
* 一緒にしたマトリクスで教授者のマトリクスにのみ枝がある場合を表す定数
*/
	private static final int LFT_EDGE_TEACHER = 3;
	
/**
* 一緒にしたマトリクスで教授者のマトリクスにのみ枝がある場合を表す定数
*/
	private static final String LFT_EDGE_STRING_TEACHER = "T";
	
/**
* 一緒にしたマトリクスで学習者のマトリクスにのみ枝がある場合を表す定数
*/
	private static final int LFT_EDGE_STUDENT = 4;
	
/**
* 一緒にしたマトリクスで学習者のマトリクスにのみ枝がある場合を表す定数
*/
	private static final String LFT_EDGE_STRING_STUDENT = "S";
	
/**
* マトリクスサイズのデフォルト値
*/
        private static final int LFT_MATRIX_VALUE = 5;

/** 
*　対象を要素とし、基準を対象を通る経路上の要素とした時の差異度を表す定数
*/
       public static final int LFT_DIFFERENCE_INDEX_OBJECT_VERTEX_STANDARD_VERTEX     = 1;

/**
*　対象を要素とし、基準を対象を通る経路上の枝とした時の差異度を表す定数
*/
        public static final int LFT_DIFFERENCE_INDEX_OBJECT_VERTEX_STANDARD_EDGE       = 2;

/**
*　対象を要素とし、基準を対象を通る経路上の始点と終点の順序対とした時の差異度を表す定数
*/
        public static final int LFT_DIFFERENCE_INDEX_OBJECT_VERTEX_STANDARD_ORDERPAIR   = 3;

/**
*　対象を枝とし、基準を対象を通る経路上の要素とした時の差異度を表す定数
*/
        public static final int LFT_DIFFERENCE_INDEX_OBJECT_EDGE_STANDARD_VERTEX       = 4;

/**
*　対象を枝とし、基準を対象を通る経路上の枝とした時の差異度を表す定数
*/
        public static final int LFT_DIFFERENCE_INDEX_OBJECT_EDGE_STANDARD_EDGE	    = 5;

/**
*　対象を枝とし、基準を対象を通る経路上の始点と終点の順序対とした時の差異度を表す定数
*/
        public static final int LFT_DIFFERENCE_INDEX_OBJECT_EDGE_STANDARD_ORDERPAIR  = 6;
        
/**
*　対象を要素の順序対とし、基準を対象を通る経路上の要素とした時の差異度を表す定数
*/
	public static final int LFT_DIFFERENCE_INDEX_OBJECT_ORDERPAIR_STANDARD_VERTEX   = 7;
       
/**
* 対象を要素の順序対とし、基準を対象を通る経路上の枝とした時の差異度を表す定数
*/	
        public static final int LFT_DIFFERENCE_INDEX_OBJECT_ORDERPAIR_STANDARD_EDGE    = 8;

/**
*　対象を要素の順序対とし、基準を対象を通る経路の始点と終点の順序対とした時の差異度を表す定数
*/
        public static final int LFT_DIFFERENCE_INDEX_OBJECT_ORDERPAIR_STANDARD_ORDERPAIR = 9;
        
/**
* 対象において可到達であることを表す定数
*/
        public static final int LFT_PREVIOUS_STANDARD   = 0;

/**
* 対象において先行となることを表す定数
*/
        public static final int LFT_REACHABLE_STANDARD = 1;
        
/**
* 設定されているデフォルトのサイズで分析を行うマトリクスを設定します。<BR>
* 教授者と学習者両方のマトリクスの要素は全て0となります。
*/
        public LFTAnalysis(){
        	BooleanMatrix baseTeacherMatrix = new BooleanMatrix(LFT_MATRIX_VALUE, LFT_MATRIX_VALUE);
        	BooleanMatrix baseStudentMatrix = new BooleanMatrix(LFT_MATRIX_VALUE, LFT_MATRIX_VALUE);
		
        	//教授者のマトリクスを設定する
		setTeacherMatrix(baseTeacherMatrix);
		
		//学習者のマトリクスを設定する
		setMatrixOfStudent(baseStudentMatrix);
	}
	
/**
* 教授者のマトリクスを設定します。<BR>
* 学習者のマトリクスは教授者のマトリクスサイズと同じで、要素は全て0となります。<BR>
* 教授者のマトリクスは正方行列である必要があります。
* @param TeacherMatrix 教授者のマトリクス
* @exception NotSquareMatrixException 教授者のマトリクスが正方行列ではない場合
*/
	public LFTAnalysis(BooleanMatrix TeacherMatrix){
		BooleanMatrix StudentMatrix =
		new BooleanMatrix(TeacherMatrix.getRowLength(), TeacherMatrix.getColLength(),
			TeacherMatrix.getRowNameArray(), TeacherMatrix.getColNameArray());
			
		//教授者のマトリクスを設定する
		setTeacherMatrix(TeacherMatrix);
			
		//学習者のマトリクスを設定する
		setMatrixOfStudent(StudentMatrix);
	}
	
/**
* 教授者のマトリクスと学習者のマトリクスを設定します。<BR>
* 教授者のマトリクスと学習者のマトリクスサイズが一致し、両者とも正方行列である必要があります。
* @param baseTeacherMatrix 教授者のマトリクス
* @param baseStudentMatrix 学習者のマトリクス
* @exception NotSquareMatrixException 教授者または学習者のマトリクスが正方行列ではない場合
* @exception DisagreementMatrixSizeException 教授者のマトリクスと学習者のマトリクスサイズが一致しない場合
*/
	public LFTAnalysis(BooleanMatrix baseTeacherMatrix, BooleanMatrix baseStudentMatrix){
		
		//教授者のマトリクスを設定する
		setTeacherMatrix(baseTeacherMatrix);
			
		//学習者のマトリクスを設定する
		setMatrixOfStudent(baseStudentMatrix);
	}
	
/**
* 教授者のマトリクスを設定します。<BR>
* 学習者のマトリクスは教授者のマトリクスサイズと同じで、要素は全て0となります。<BR>
* 教授者のマトリクスは正方行列である必要があります。
* @param TeacherMatrix 教授者のマトリクス
* @exception NotSquareMatrixException 教授者のマトリクスが正方行列ではない場合
*/
	public LFTAnalysis(ArithmeticMatrix TeacherMatrix){
		BooleanMatrix baseStudentMatrix =
		new BooleanMatrix(TeacherMatrix.getRowLength(), TeacherMatrix.getColLength(),
			TeacherMatrix.getRowNameArray(), TeacherMatrix.getColNameArray());
			
		//教授者のマトリクスを設定する
		setTeacherMatrix(TeacherMatrix.toBooleanMatrix());
			
		//学習者のマトリクスを設定する
		setMatrixOfStudent(baseStudentMatrix);
	}
	
/**
* 教授者のマトリクスと学習者のマトリクスを設定します。<BR>
* 教授者のマトリクスと学習者のマトリクスサイズが一致し、両者とも正方行列である必要があります。
* @param TeacherMatrix 教授者のマトリクス
* @param StudentMatrix 学習者のマトリクス
* @exception NotSquareMatrixException 教授者または学習者のマトリクスが正方行列ではない場合
* @exception DisagreementMatrixSizeException 教授者のマトリクスと学習者のマトリクスサイズが一致しない場合
*/
	public LFTAnalysis(ArithmeticMatrix TeacherMatrix, ArithmeticMatrix StudentMatrix){
		
		//教授者のマトリクスを設定する
		setTeacherMatrix(TeacherMatrix.toBooleanMatrix());
			
		//学習者のマトリクスを設定する
		setMatrixOfStudent(StudentMatrix.toBooleanMatrix());
	}
	
/**
* Boolean型に変換する際のしきい値と教授者のマトリクスを設定します。<BR>
* 学習者のマトリクスは教授者のマトリクスサイズと同じで、要素は全て0となります。<BR>
* 教授者のマトリクスは正方行列である必要があります。
* @param TeacherMatrix 教授者のマトリクス
* @param value BooleanMatrixに変換する際のしきい値
* @exception NotSquareMatrixException 教授者のマトリクスが正方行列ではない場合
*/
	public LFTAnalysis(ArithmeticMatrix TeacherMatrix, double value){
		BooleanMatrix baseStudentMatrix =
		new BooleanMatrix(TeacherMatrix.getRowLength(), TeacherMatrix.getColLength(),
			TeacherMatrix.getRowNameArray(), TeacherMatrix.getColNameArray());
			
		//教授者のマトリクスを設定する
		setTeacherMatrix(TeacherMatrix.toBooleanMatrix());
			
		//学習者のマトリクスを設定する
		setMatrixOfStudent(baseStudentMatrix);
	}
	
/**
* Boolean型に変換する際のしきい値と教授者のマトリクス、学習者のマトリクスを設定します。<BR>
* 教授者のマトリクスと学習者のマトリクスサイズが一致し、両者とも正方行列である必要があります。
* @param TeacherMatrix 教授者のマトリクス
* @param StudentMatrix 学習者のマトリクス
* @param value BooleanMatrixに変換する際のしきい値
* @exception NotSquareMatrixException 教授者または学習者のマトリクスが正方行列ではない場合
* @exception DisagreementMatrixSizeException 教授者のマトリクスと学習者のマトリクスサイズが一致しない場合
*/
	public LFTAnalysis(ArithmeticMatrix TeacherMatrix, ArithmeticMatrix StudentMatrix, double value){
		
		//教授者のマトリクスを設定する
		setTeacherMatrix(TeacherMatrix.toBooleanMatrix());
			
		//学習者のマトリクスを設定する
		setMatrixOfStudent(StudentMatrix.toBooleanMatrix());
	}

/**
*隣接マトリクスを元に、要素vjを対象とし、vjにおいて先行または可到達である有向枝の集合を求めます。
*@param Matrix 調べる隣接マトリクス
*@param vj 対象とする要素
*@param LFT_SET_STANDARD 集合を表す定数
*@return 要素vjを対象とした有向枝の先行または可到達集合
*@exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
*/
    public EdgeSet getEdgeSet(BooleanMatrix matrix, Vertex vj,int LFT_STANDARD){
    	if(matrix.isSquare()){
    		int lft = LFT_STANDARD;
    		if(lft == 0){	
				//可到達マトリクスを生成
				BooleanMatrix lookMatrix = new BooleanMatrix(matrix.getRowLength(), matrix.getColLength(),matrix.getRowNameArray(), matrix.getColNameArray());
				lookMatrix.matrixCopy(matrix.getReachMatrix());
				
				EdgeSet edgeHash = new EdgeSet();
				int v = vj.getVertex();
				int x = 0;	  
				for(int i=0; i<lookMatrix.getRowLength(); i++){
					for(int j=0; j<lookMatrix.getColLength(); j++){
						if(lookMatrix.getCell(i, v) == LFT_EDGE_FALSE){
							lookMatrix.setCell(i,j,false);
						}
					}
				}
				for(int i=0; i<lookMatrix.getRowLength(); i++){
					for(int j=0; j<lookMatrix.getColLength(); j++){
						if(lookMatrix.getCell(i, v) == LFT_EDGE_FALSE){
							lookMatrix.setCell(j,i,false);
						}
					}
				}
	    
				for(int i=0; i<lookMatrix.getRowLength(); i++){
					if(lookMatrix.getCell(i,v) == LFT_EDGE_FALSE){
						lookMatrix.setCell(v,i,false);
					}
				}		   
				//隣接マトリクスと可到達マトリクスから枝を抽出
				for(int i=0; i<lookMatrix.getRowLength(); i++){
					for(int j=0; j<lookMatrix.getColLength(); j++){
						if(matrix.getCell(i,j) && lookMatrix.getCell(i,j) == LFT_EDGE_TRUE){
							edgeHash.add(new Edge(i,j));
						} 
					}
				}
				return edgeHash;
    		}
    		
    		else if(lft==1){
				//可到達マトリクスを生成
				BooleanMatrix look2Matrix = new BooleanMatrix(matrix.getRowLength(),matrix.getColLength(),matrix.getRowNameArray(),matrix.getColNameArray());
				look2Matrix.matrixCopy(matrix.getReachMatrix());
    
				EdgeSet edge2Hash = new EdgeSet();
				int v2 = vj.getVertex();
				for(int i=0; i<look2Matrix.getColLength(); i++){
					for(int j=0; j<look2Matrix.getRowLength(); j++){
						if(look2Matrix.getCell(v2,i) == LFT_EDGE_FALSE){
							look2Matrix.setCell(i,j,false);
						}
					}
				}    	    
				for(int i=0; i<look2Matrix.getRowLength(); i++){
					for(int j=0; j<look2Matrix.getColLength(); j++){
						if(look2Matrix.getCell(i,j) && matrix.getCell(i,j) == LFT_EDGE_TRUE){
							edge2Hash.add(new Edge(i,j));
						}
					}	
				}
				return edge2Hash;
				
			}
    	}
    	else{
    		throw new NotSquareMatrixException();
    	}
		return null;
    }
	//	return null;
    


    
    
    
    
    
/**
*隣接マトリクスを元に、有向枝edを対象とし、edにおいて先行または可到達である有向枝の集合を求めます。
*@param matrix 調べる隣接マトリクス
*@param ed 対象とする有向枝
*@param LFT_STANDARD 集合を表す定数
*@return 有向枝を対象とした有向枝の集合
*@exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
*/
    public EdgeSet getEdgeSet(BooleanMatrix matrix, Edge ed, int LFT_STANDARD){
    	if(matrix.isSquare()){
    		int lft = LFT_STANDARD;
    		if(lft == 0){
    			EdgeSet edgeHash = new EdgeSet();
    			if(matrix.getCell(ed.getBegin(), ed.getEnd()) == LFT_EDGE_TRUE){
    				edgeHash = getEdgeSet(matrix, new Vertex(ed.getEnd()),1);		
    			}
    			return edgeHash;	    
    		}
    		else if(lft == 1){	
    			EdgeSet edHash = new EdgeSet();
    			if(matrix.getCell(ed.getBegin(), ed.getEnd()) == LFT_EDGE_TRUE){
    				edHash = getEdgeSet(matrix,new Vertex(ed.getBegin()),0);
    			}
    			return edHash;
    		}
    	}

    	else{
    		throw new NotSquareMatrixException();
    	}
		return null;
    }
    
    

/**
*隣接マトリクスを元にある要素を通過する経路の集合に含まれる有向枝の集合を求めます。
*@param baseMatrix 調べる隣接マトリクス
*@param vi 通過する要素
*@return 経路の集合に含まれる枝の集合
*@exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
*/
    private EdgeSet passElementGetEdgeSet(BooleanMatrix baseMatrix, Vertex vi){
       	if(baseMatrix.isSquare()){
	    //可到達マトリクスを生成
	    BooleanMatrix lookMatrix = new BooleanMatrix(baseMatrix.getRowLength(),baseMatrix.getColLength(),baseMatrix.getRowNameArray(),baseMatrix.getColNameArray());
	    lookMatrix.matrixCopy(baseMatrix.getReachMatrix());
	    
	    EdgeSet edgeHash = new EdgeSet();
	    int v = vi.getVertex();
	    int count1 = 0;
	    int count2 = 0;
	    for(int i=0; i<baseMatrix.getRowLength(); i++){
		for(int j=0; j<baseMatrix.getColLength(); j++){
		    if(baseMatrix.getCell(i,v) == LFT_EDGE_TRUE){
			count1++;
			if(v!=j){
			    baseMatrix.setCell(i,j,false);
			}
		    }	
		}
	    }
	    for(int i=0; i<baseMatrix.getRowLength(); i++){
		for(int j=0; j<baseMatrix.getColLength(); j++){
		    if(baseMatrix.getCell(v,i) == LFT_EDGE_TRUE){
			count2++;
			if(v!=j){
			    baseMatrix.setCell(j,i,false);
			}
		    }
		}
	    }
	    if(count1>0 && count2>0){
		for(int i=0; i<lookMatrix.getRowLength(); i++){
		    for(int j=0; j<lookMatrix.getColLength(); j++){
			if(lookMatrix.getCell(i,j) && baseMatrix.getCell(i,j) == LFT_EDGE_TRUE){
			    edgeHash.add(new Edge(i,j));
			}
		    }
		}
	    }     	
	    return edgeHash;
	}
	else{
	    throw new NotSquareMatrixException();
	}
    }

/**
*隣接マトリクスを元に、対象を要素vjとし、vjにおいて先行または可到達である要素の集合を求めます。
*@param matrix 調べる隣接マトリクス
*@param vj 対象となる要素
*@param LFT_STANDARD 集合を表す定数
*@exception NotSquareMatrixException 設定したマトリクスが正方行列でない場合
*/
    public VertexSet getVertexSet(BooleanMatrix matrix, Vertex vj, int LFT_STANDARD){
    	if(matrix.isSquare()){
    		int lft = LFT_STANDARD;
    		if(lft == 0){
    			//可到達マトリクスの生成
    			BooleanMatrix lookMatrix = new BooleanMatrix(matrix.getRowLength(),matrix.getColLength(),matrix.getRowNameArray(),matrix.getColNameArray());
    			lookMatrix.matrixCopy(matrix.getReachMatrix());
	    
    			VertexSet treeSet = new VertexSet();
    			int v = vj.getVertex();
    			for(int i=0; i<lookMatrix.getRowLength(); i++){
    				if(lookMatrix.getCell(i,v) == LFT_EDGE_TRUE){
    					if(i!=v){
    						treeSet.add(new Vertex(i));
    					}
    				}
    			}
    			return treeSet;
    		}
    		else if(lft == 1){
    			//可到達マトリクスの生成
    			BooleanMatrix look2Matrix = new BooleanMatrix(matrix.getRowLength(),matrix.getColLength(),matrix.getRowNameArray(),matrix.getColNameArray());
    			look2Matrix.matrixCopy(matrix.getReachMatrix());
	   
    			VertexSet ptreeSet = new VertexSet();
    			int v1 = vj.getVertex();
    			for(int i=0; i<look2Matrix.getRowLength(); i++){
    				if(look2Matrix.getCell(v1,i) == LFT_EDGE_TRUE){
    					if(i!=v1){
    						ptreeSet.add(new Vertex(i));
    					}
    				}
    			}
    			return ptreeSet;
    		}
    	}
    	else{
    		throw new NotSquareMatrixException();
    	}
		return null;
    }
    
    
    
    
    
    
/**
*隣接マトリクスを元に、対象を有向枝edとし、edにおいて先行または可到達である要素の集合を求めます。
*@param matrix 調べる隣接マトリクス
*@param ed 対象とする有向枝
*@param LFT_STANDARD 集合を表す定数
*@return 有向枝を対象とする経路内に含まれる要素の集合
*@exception NotSquareMatrixException 設定したマトリクスが正方行列でない場合
*/ 
    public VertexSet getVertexSet(BooleanMatrix matrix, Edge ed, int LFT_STANDARD){
    	if(matrix.isSquare()){
    		int lft = LFT_STANDARD;
    		if(lft == 0){
    			VertexSet treeSet = new VertexSet();
    			if(matrix.getCell(ed.getBegin(), ed.getEnd()) == LFT_EDGE_TRUE){
    				treeSet = getVertexSet(matrix,new Vertex(ed.getBegin()),0);
    			}
    			return treeSet;
    		}
       	
    		else if(lft == 1){
    			VertexSet ptreeSet = new VertexSet();
    			if(matrix.getCell(ed.getBegin(), ed.getEnd()) == LFT_EDGE_TRUE){
    				ptreeSet = getVertexSet(matrix, new Vertex(ed.getEnd()),1);
    			}
    			return ptreeSet;
    		}
    	}
    	else{
    		throw new NotSquareMatrixException();
    	}
		return null;
    }

/**
*隣接マトリクスを元に、要素の順序対opを対象とし、opの経路の集合に含まれる要素の集合を求めます。
*@param matrix 調べる隣接マトリクス
*@param op 対象とする順序対
*@return 要素の順序対に含まれる要素の集合
*@exception  NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
*/
    public VertexSet getVertexSet(BooleanMatrix matrix, OrderPair op){
	if(matrix.isSquare()){
	    VertexSet treeSet = new VertexSet();
	    VertexSet iSet = new VertexSet();
	    VertexSet jSet = new VertexSet();
	    //可到達マトリクスを生成
	    BooleanMatrix lookMatrix = new BooleanMatrix(matrix.getRowLength(), matrix.getColLength(), matrix.getRowNameArray(), matrix.getColNameArray());
	    lookMatrix.matrixCopy(matrix.getReachMatrix());
	    
	    if(lookMatrix.getCell(op.getBegin(), op.getEnd()) == LFT_EDGE_TRUE){
		for(int i=0; i<lookMatrix.getColLength(); i++){
		    if(lookMatrix.getCell(op.getBegin(), i) == LFT_EDGE_TRUE){
			iSet.add(new Double(i));
		    }   			
		}
		for(int j=0; j<lookMatrix.getRowLength(); j++){
		    if(lookMatrix.getCell(j, op.getEnd()) == LFT_EDGE_TRUE){
			jSet.add(new Double(j));
		    }
		}
		Iterator it = jSet.iterator();
		while(it.hasNext()){
		    Double dy = (Double)it.next();
		    if(iSet.contains(dy) == true){
			treeSet.add(dy);
		    }
		}
		treeSet.remove(new Double(op.getBegin()));
		treeSet.remove(new Double(op.getEnd()));
	    }	
	    return treeSet;
       	}
       	else{
	    throw new NotSquareMatrixException();
       	}
    }
       
/**
*隣接マトリクスを元に、要素vkを対象とし、vkを通過する経路の集合に含まれる要素の集合を求めます。
*@param nextMatrix 調べる隣接マトリクス
*@param vk 有向枝の通過点
*@return 経路の集合に含まれる要素の集合
*@exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
*/
    private Set passElementGetElementSet(BooleanMatrix nextMatrix, int vk){
       	if(nextMatrix.isSquare()){
	    //可到達マトリクスの生成
	    BooleanMatrix lookMatrix = new BooleanMatrix(nextMatrix.getRowLength(),nextMatrix.getColLength(),nextMatrix.getRowNameArray(),nextMatrix.getColNameArray());
	    lookMatrix.matrixCopy(nextMatrix.getReachMatrix());
	    Set treeSet = new TreeSet();
	    int v = vk;
	    int count1 = 0;
	    int count2 = 0;
	    for(int i=0; i<lookMatrix.getRowLength(); i++){
		if(lookMatrix.getCell(v,i) == LFT_EDGE_TRUE){
		    treeSet.add(new Double(i));  						
		    if(v!=i){
			count1++;
		    }
		}
	    }
	    for(int i=0; i<lookMatrix.getColLength(); i++){
		if(lookMatrix.getCell(i,v) == LFT_EDGE_TRUE){
		    treeSet.add(new Double(i)); 
		    if(v!=i){
			count2++;
		    }
		}
	    }
	    if(count1==0 || count2==0){
		treeSet.clear();
	    }
	    return treeSet;
       	}
       	else{
	    throw new NotSquareMatrixException();
       	}
    }
       
/**
*隣接マトリクスを元に、要素vを対象とし、vを通過する要素の順序対の集合を求めます。
*@param matrix 調べる隣接マトリクス
*@param v有向枝の通過点
*@return vを通過する要素の順序対集合
*@exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
*/       
    public OrderPairSet getOrderPairSet(BooleanMatrix matrix, Vertex v){
       	if(matrix.isSquare()){
	    //可到達マトリクスを生成
	    BooleanMatrix lookMatrix = new BooleanMatrix(matrix.getRowLength(),matrix.getColLength(),matrix.getRowNameArray(),matrix.getColNameArray());
	    lookMatrix.matrixCopy(matrix.getReachMatrix());
	    int vk = v.getVertex();
	    OrderPairSet edgeHash = new OrderPairSet();
	    int count1=0;
	    for(int i=0; i<lookMatrix.getColLength(); i++){
		if(lookMatrix.getCell(vk,i) == LFT_EDGE_TRUE){
		    if(vk!=i){
			count1++;	       		
		    }
		}
	    }
	    int[] cou1 = new int[count1];        		
	    
	    int x=0;   
	    for(int i=0; i<lookMatrix.getColLength(); i++){
		if(lookMatrix.getCell(vk,i) == LFT_EDGE_TRUE){
		    if(vk!=i){
			cou1[x] = i;
			x++;
		    }
		}
	    }
       
	    int count2=0;
	    for(int i=0; i<lookMatrix.getColLength(); i++){
		if(lookMatrix.getCell(i,vk) == LFT_EDGE_TRUE){
		    if(vk!=i){
			count2++;	       					
		    }
		}
	    }
	    int[] cou2 = new int[count2];        		
	    
	    int y=0;   
	    for(int i=0; i<lookMatrix.getColLength(); i++){
		if(lookMatrix.getCell(i,vk) == LFT_EDGE_TRUE){
		    if(vk!=i){
			cou2[y] = i;
			y++;
		    }
		}
	    }
	    for(int i=0; i<lookMatrix.getColLength(); i++){
		if(lookMatrix.getCell(vk,i)==true){
		    if(i!=vk){
			edgeHash.add(new Edge(vk,i));
		    }
		}
	    }
	    for(int j=0; j<lookMatrix.getRowLength(); j++){
		if(lookMatrix.getCell(j,vk) == true){
		    if(j!=vk){
			edgeHash.add(new Edge(j,vk));
		    }
		}
	    }

	    for(int i=0; i<count1; i++){
		for(int j=0; j<count2; j++){
		    edgeHash.add(new Edge(cou1[i],cou2[j]));
		}
	    }     		
	    return edgeHash;
       	}
       	else{
	    throw new NotSquareMatrixException();
       	}
    }
       
/**
*隣接マトリクスを元に、要素の順序対opを対象とし、opの経路内に含まれる有向枝の集合を求めます。
*@param matrix 調べる隣接マトリクス
*@param op 対象とする要素の順序対
*@return 経路内に含まれる有向枝の集合 
*@exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
*/
    public EdgeSet getEdgeSet(BooleanMatrix matrix, OrderPair op){
       	if(matrix.isSquare()){
	    //可到達マトリクスを生成
	    BooleanMatrix lookMatrix = new BooleanMatrix(matrix.getRowLength(), matrix.getColLength(), matrix.getRowNameArray(), matrix.getColNameArray());
	    lookMatrix.matrixCopy(matrix.getReachMatrix());
       	    
	    EdgeSet edgeHash = new EdgeSet();
	    //始点の枝修正
	    for(int j=0; j<lookMatrix.getColLength(); j++){
		if(lookMatrix.getCell(op.getBegin(),j) == LFT_EDGE_FALSE){
		    for(int s=0; s<lookMatrix.getColLength(); s++){
			lookMatrix.setCell(j,s,false);
		    }
		}
	    }
	    //終点の枝修正
	    for(int j=0; j<lookMatrix.getColLength(); j++){
		if(lookMatrix.getCell(op.getEnd(),j) == LFT_EDGE_TRUE){
		    for(int s=0; s<lookMatrix.getColLength(); s++){
			lookMatrix.setCell(j,s,false);
		    }
		}
	    }
	    for(int i=0; i<lookMatrix.getRowLength(); i++){
		if(lookMatrix.getCell(i,op.getEnd()) == LFT_EDGE_FALSE){
		    if(i!=op.getEnd()){
			for(int s=0; s<lookMatrix.getColLength(); s++){
			    lookMatrix.setCell(i,s,false);
			    lookMatrix.setCell(s,i,false);
			}
		    }
		}
	    }
	    //枝が存在するかどうか
	    for(int i=0; i<matrix.getRowLength(); i++){
		for(int j=0; j<matrix.getColLength(); j++){
		    if(matrix.getCell(i,j) && lookMatrix.getCell(i,j) == LFT_EDGE_TRUE){ 
			edgeHash.add(new Edge(i,j));
		    }
		}
	    }
	    return edgeHash;
       	}
       	else{
	    throw new NotSquareMatrixException();
       	}
    }       

/**隣接マトリクスを元に、要素の順序対opを対象とし、opの経路内に含まれる要素の順序対集合を求めます。
*@param matrix 調べる隣接マトリクス
*@param op 対象とする要素の順序対
*@return 経路内に含まれる要素の順序対集合(O<vi→vj>) 
*@exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
*/
    public OrderPairSet getOrderPairSet(BooleanMatrix matrix, OrderPair op){
	if(matrix.isSquare()){
	    
	    //可到達マトリクスを生成
	    BooleanMatrix lookMatrix = new BooleanMatrix(matrix.getRowLength(), matrix.getColLength(), matrix.getRowNameArray(), matrix.getColNameArray());
	    lookMatrix.matrixCopy(matrix.getReachMatrix());
	    
	    OrderPairSet edgeHash = new OrderPairSet();
	    
	    //始点修正
	    for(int j=0; j<lookMatrix.getColLength(); j++){
		if(lookMatrix.getCell(op.getBegin(),j) == LFT_EDGE_FALSE){
		    for(int s=0; s<lookMatrix.getColLength(); s++){
			lookMatrix.setCell(j,s,false);
			lookMatrix.setCell(s,j,false);
		    }
		}
	    }
	    //終点修正
	    for(int i=0; i<lookMatrix.getColLength(); i++){
		if(lookMatrix.getCell(i,op.getEnd()) == LFT_EDGE_FALSE){
		    for(int s=0; s<lookMatrix.getColLength(); s++){
			lookMatrix.setCell(i,s,false);
			lookMatrix.setCell(s,i,false);
		    }
		}
		lookMatrix.setCell(op.getEnd(),i,false);
	    }
	    
	    for(int i=0; i<lookMatrix.getRowLength(); i++){
		for(int j=0; j<lookMatrix.getColLength(); j++){
		    if(i==j){
			lookMatrix.setCell(i,j,false);
		    }
		}
	    }
	    
	    for(int i=0; i<lookMatrix.getRowLength(); i++){
		for(int j=0; j<lookMatrix.getColLength(); j++){
		    if(lookMatrix.getCell(i,j) == LFT_EDGE_TRUE){ 
			edgeHash.add(new Edge(i,j));
		    }
		}
	    }
	    return edgeHash;
	}
	else{
	    throw new NotSquareMatrixException();
	}
    }         
       
/**指定した対象と基準においての差異度を返します。
*@param LFT_DIFFERENCE_INDEX 対象と基準を表す定数
*@return 差異度
*/
    public double getDifferenceIndex(int LFT_DIFFERENCE_INDEX){
    	int lft = LFT_DIFFERENCE_INDEX;
    	if(lft == LFT_DIFFERENCE_INDEX_OBJECT_EDGE_STANDARD_VERTEX){
    		lft = LFT_DIFFERENCE_INDEX_OBJECT_VERTEX_STANDARD_EDGE;
    	}
    	if(lft == LFT_DIFFERENCE_INDEX_OBJECT_ORDERPAIR_STANDARD_VERTEX){
    		lft = LFT_DIFFERENCE_INDEX_OBJECT_VERTEX_STANDARD_ORDERPAIR; 	    		
    	}
    	
    	
    	switch(lft)
		{    	
		case LFT_DIFFERENCE_INDEX_OBJECT_VERTEX_STANDARD_VERTEX:
			double denominator = 0.0; //分母
			double numerator = 0.0; //分子
			VertexSet teach1Set = new VertexSet();
			VertexSet stude1Set = new VertexSet();
			VertexSet teach2Set = new VertexSet();
			VertexSet stude2Set = new VertexSet();
			VertexSet in1Set = new VertexSet();
			VertexSet out1Set = new VertexSet();
			VertexSet in2Set = new VertexSet();
			VertexSet out2Set = new VertexSet();
	
			for(int i=0; i<teacherMatrix.getRowLength(); i++){
				double ns = 0;
				double ds = 0;
				teach1Set = getVertexSet(teacherMatrix, new Vertex(i),  0);
				stude1Set = getVertexSet(studentMatrix, new Vertex(i),  0);
				teach2Set = getVertexSet(teacherMatrix, new Vertex(i),  1);
				stude2Set = getVertexSet(studentMatrix, new Vertex(i),  1);
				//分子
				in1Set = nablaSet(teach1Set,stude1Set);
				out1Set = nablaSet(teach2Set,stude2Set);
				ns = (double)orSetSize(in1Set, out1Set);
				numerator = ns + numerator;
				//分母
				in2Set = orSet(teach1Set, stude1Set);
				out2Set = orSet(teach2Set, stude2Set);
				ds = (double)orSetSize(in2Set, out2Set);
	 
				denominator = ds + denominator;
				teach1Set.clear();
				stude1Set.clear();
				teach2Set.clear();
				stude2Set.clear();
				in1Set.clear();
				out1Set.clear();
				in2Set.clear();
				out2Set.clear();
			}
			return numerator/denominator;
    
   
	 
			
		case LFT_DIFFERENCE_INDEX_OBJECT_VERTEX_STANDARD_EDGE: 	
			double denominator2 = 0.0; //分母
			double numerator2 = 0.0; //分子
			EdgeSet teach21Set = new EdgeSet();
			EdgeSet stude21Set = new EdgeSet();
			EdgeSet in21Set = new EdgeSet();
			EdgeSet teach22Set = new EdgeSet();
			EdgeSet stude22Set = new EdgeSet();
			EdgeSet out21Set = new EdgeSet();
			EdgeSet in22Set = new EdgeSet();
			EdgeSet out22Set = new EdgeSet();
	
			for(int i=0; i<teacherMatrix.getRowLength(); i++){
				double ns = 0;
				double ds = 0;
				//分子
				teach21Set = getEdgeSet(teacherMatrix, new Vertex(i),0);
				stude21Set = getEdgeSet(studentMatrix, new Vertex(i),0);
				in21Set = nablaEdgeSet(teach21Set, stude21Set);
				teach22Set = getEdgeSet(teacherMatrix, new Vertex(i),1);
				stude22Set = getEdgeSet(studentMatrix, new Vertex(i),1);
				out21Set = nablaEdgeSet(teach22Set, stude22Set);
				ns = (double)orEdgeSetSize(in21Set,out21Set);
				numerator2 = ns + numerator2;	
				//分母
				in22Set = orEdgeSet(teach21Set, stude21Set);
				out22Set = orEdgeSet(teach22Set, stude22Set);
	    
				ds = (double)orEdgeSetSize(in22Set,out22Set);		
				denominator2 = ds + denominator2;			
				teach21Set.clear();
				teach22Set.clear();
				stude21Set.clear();
				stude22Set.clear();
				in21Set.clear();
				out21Set.clear();
				in22Set.clear();
				out22Set.clear();			
			}	
			return numerator2/denominator2;
			
			
			
			
		case LFT_DIFFERENCE_INDEX_OBJECT_VERTEX_STANDARD_ORDERPAIR: 	
			double denominator3 = 0.0; //分母
			double numerator3 = 0.0; //分子
			OrderPairSet tedge = new OrderPairSet();
			OrderPairSet sedge = new OrderPairSet();
			OrderPairSet sameedge = new OrderPairSet(); 
			
			for(int i=0; i<teacherMatrix.getRowLength(); i++){
				double t = 0;
				double s = 0;
				double ts = 0;       
				double nume = 0;
				double deno = 0;
				Vertex ver = new Vertex(i);
				tedge = getOrderPairSet(teacherMatrix, ver);
				sedge = getOrderPairSet(studentMatrix, ver);
				t = tedge.size();
				s = sedge.size();	
		    
				Iterator ite = sedge.iterator();
				while(ite.hasNext()){
					Edge ed = (Edge)ite.next(); 
					if(tedge.contains(ed) == true){
						sameedge.add(ed);
					}
				}
				ts = sameedge.size();
				nume = t - ts + s - ts;
				numerator3 = nume + numerator3;
				deno = t + s -ts;
				denominator3 = deno + denominator3;
				tedge.clear();
				sedge.clear();
				sameedge.clear(); 
	       		}
			return numerator3/denominator3;
		
	
		
		
		
		case LFT_DIFFERENCE_INDEX_OBJECT_EDGE_STANDARD_EDGE:
			double denominator5 = 0.0; //分母
	       	double numerator5 = 0.0; //分子
	     	EdgeSet teach51Set = new EdgeSet();
	       	EdgeSet stude51Set = new EdgeSet();
	       	EdgeSet teach52Set = new EdgeSet();
	       	EdgeSet stude52Set = new EdgeSet();
	       	EdgeSet in51Set = new EdgeSet();
	       	EdgeSet out51Set = new EdgeSet();
	       	EdgeSet in52Set = new EdgeSet();
	       	EdgeSet out52Set = new EdgeSet();
	       	
	       	for(int i=0; i<teacherMatrix.getRowLength(); i++){
	       		for(int j=0; j<teacherMatrix.getRowLength(); j++){
	       			double ns = 0;	
	       			double ds = 0;
	       			teach51Set = getEdgeSet(teacherMatrix, new Edge(i,j),0);
	       			stude51Set = getEdgeSet(studentMatrix, new Edge(i,j),0);
	       			teach52Set = getEdgeSet(teacherMatrix, new Edge(i,j),1);
	       			stude52Set = getEdgeSet(studentMatrix, new Edge(i,j),1);
	       			//分子
	       			in51Set = nablaEdgeSet(teach51Set, stude51Set);
	       			out51Set = nablaEdgeSet(teach52Set, stude52Set);
	       			ns = (double)orEdgeSetSize(in51Set, out51Set);
	       			//分母
	       			in52Set = orEdgeSet(teach51Set, stude51Set);
	       			out52Set = orEdgeSet(teach52Set, stude52Set);
	       			ds = (double)orEdgeSetSize(in52Set, out52Set);
	       			numerator5 = ns + numerator5;
	       			denominator5 = ds + denominator5;
	       			teach51Set.clear();
	       			teach52Set.clear();
	       			stude51Set.clear();
	       			stude52Set.clear();
	       			in51Set.clear();
	       			in52Set.clear();
	       			out51Set.clear();
	       			out52Set.clear();
	       		}
	       	}
	       	return numerator5/denominator5;	
			
			
			
	       	
		case LFT_DIFFERENCE_INDEX_OBJECT_EDGE_STANDARD_ORDERPAIR:
			double denominator6 = 0.0; //分母 
			double numerator6 = 0.0; //分子
			EdgeSet teach6Set = new EdgeSet();
			EdgeSet stude6Set = new EdgeSet();
			for(int i=0; i<teacherMatrix.getRowLength(); i++){
			    for(int j=0; j<teacherMatrix.getRowLength(); j++){
			    	double ns = 0;	
			    	double ds = 0;
			    	teach6Set = getVertexPairSet(teacherMatrix, i, j);
			    	stude6Set = getVertexPairSet(studentMatrix, i, j);
			    	//分子
			    	ns = (double)nablaEdgeSetSize(teach6Set, stude6Set);
			    	//分母
			    	ds = (double)orEdgeSetSize(teach6Set, stude6Set);
			    	numerator6 = ns + numerator6;
			    	denominator6 = ds + denominator6;
			    	teach6Set.clear();
			    	stude6Set.clear();
			    }
			}
			return numerator6/denominator6;
			
			
			
					
		case LFT_DIFFERENCE_INDEX_OBJECT_ORDERPAIR_STANDARD_EDGE:
			double denominator8 = 0.0; //分母 
			double numerator8 = 0.0;  //分子
		       	EdgeSet teach8Set = new EdgeSet();
		       	EdgeSet stude8Set = new EdgeSet();
		       	
		       	for(int i=0; i<teacherMatrix.getRowLength(); i++){
		       		for(int j=0; j<teacherMatrix.getRowLength(); j++){
		       			double ns = 0;	
		       			double ds = 0;
		       			//分子
		       			teach8Set = getEdgeSet(teacherMatrix, new OrderPair(i,j));
		       			stude8Set = getEdgeSet(studentMatrix, new OrderPair(i,j));
		       			ns = (double)nablaEdgeSetSize(teach8Set, stude8Set);		
		       			//分母
		       			ds = (double)orEdgeSetSize(teach8Set, stude8Set);
				
		       			numerator8 = ns + numerator8;
		       			denominator8 = ds + denominator8;
		       			teach8Set.clear();
		       			stude8Set.clear();
		       		}
		       	}
		       	return numerator8/denominator8;
		
		       	
		       	
		       	
		case LFT_DIFFERENCE_INDEX_OBJECT_ORDERPAIR_STANDARD_ORDERPAIR:
		 	double denominator9 = 0.0; //分母
	       	double numerator9 = 0.0; //分子
	       	OrderPairSet teach9Set = new OrderPairSet();
	       	OrderPairSet stude9Set = new OrderPairSet();
		for(int i=0; i<teacherMatrix.getRowLength(); i++){
		    for(int j=0; j<teacherMatrix.getRowLength(); j++){
			double ns = 0;	
			double ds = 0;
			//分子
			teach9Set = getOrderPairSet(teacherMatrix, new OrderPair(i,j));
			stude9Set = getOrderPairSet(studentMatrix, new OrderPair(i,j));
			ns = (double)nablaOrderPairSetSize(teach9Set, stude9Set);		
			//分母
			ds = (double)orOrderPairSetSize(teach9Set, stude9Set);
			
			numerator9 = ns + numerator9;
			denominator9 = ds + denominator9;
			teach9Set.clear();
			stude9Set.clear();
		    }
	       	}
	       	return numerator9/denominator9;
		
		}
		return LFT_DIFFERENCE_INDEX;
	 }

/**コレクションSetに対応している2つの集合から∇Setを返します。  
*@return ∇の集合       
*/     
    private VertexSet nablaSet(VertexSet teacherSet, VertexSet studentSet){
	VertexSet sameSet = new VertexSet();
	Iterator it = teacherSet.iterator();
	while(it.hasNext()){
	    Vertex dy = (Vertex)it.next();
	    if(studentSet.contains(dy) == true){
		sameSet.add(dy);
	    }
	    studentSet.add(dy);
	}
	studentSet.removeAll(sameSet);
	return studentSet;
    }
          
/**コレクションSetに対応している2つの集合からnablaの大きさを返します。
*@return ∇集合のサイズ
*/ 
    private int nablaSetSize(VertexSet teacherSet, VertexSet studentSet){
      	int size = 0;
      	Set nablaSet = new TreeSet();
      	nablaSet = nablaSet(teacherSet, studentSet);
      	size = nablaSet.size();
      	return size;
    }
      
/**コレクションSetに対応している２つの集合からorSetを返します。
*@return orの集合
*/       
    private VertexSet orSet(VertexSet beforeSet, VertexSet afterSet){
      	Iterator ite = beforeSet.iterator();
      	while(ite.hasNext()){
	    Vertex dy = (Vertex)ite.next();
	    afterSet.add(dy);
      	}
      	return afterSet;
    }
      
/**コレクションSetに対応している２つの集合から論理和の大きさを返します。 
*@return or集合のサイズ
*/       
    private int orSetSize(VertexSet beforeSet, VertexSet afterSet){
      	int size = 0;
      	Set orSet = new TreeSet();
      	orSet = orSet(beforeSet, afterSet);
      	size = orSet.size();
      	return size;
    }
   
/**EdgeSetに対応している2つの集合から∇Setを返します。
*@return ∇の集合
*/  
    private EdgeSet nablaEdgeSet(EdgeSet teacherSet, EdgeSet studentSet){
	EdgeSet sameEdgeSet = new EdgeSet();
	Iterator it = teacherSet.iterator();
	while(it.hasNext()){
	    Edge ed = (Edge)it.next();
	    if(studentSet.contains(ed) == true){
		sameEdgeSet.add(ed);
	    }
	    studentSet.add(ed);
	}
	Iterator it2 = sameEdgeSet.iterator();
	while(it2.hasNext()){
	    Edge ed = (Edge)it2.next();
	    studentSet.remove(ed);
	}
	return studentSet;
    }
      
/**EdgeSetに対応している2つの集合から∇の大きさを返します。
*@return ∇集合のサイズ
*/ 
    private int nablaEdgeSetSize(EdgeSet teacherEdgeSet, EdgeSet studentEdgeSet){
      	int size = 0;
      	EdgeSet lEdgeSet = new EdgeSet();
      	lEdgeSet = nablaEdgeSet(teacherEdgeSet, studentEdgeSet);
      	size = lEdgeSet.size();
      	return size;
    }
/**EdgeSetに対応している２つの集合からorSetを返します。
*@return orの集合
*/       
    private EdgeSet orEdgeSet(EdgeSet beforeEdgeSet, EdgeSet afterEdgeSet){
      	Iterator ite = beforeEdgeSet.iterator();
      	while(ite.hasNext()){
	    Edge ed = (Edge)ite.next();
	    afterEdgeSet.add(ed);
      	}
      	return afterEdgeSet;
    }
      
/**EdgeSetに対応している２つの集合から論理和の大きさを返します。 
*@return or集合のサイズ
*/       
    private int orEdgeSetSize(EdgeSet beforeEdgeSet, EdgeSet afterEdgeSet){
      	int size = 0;
      	EdgeSet oEdgeSet = new EdgeSet();
      	oEdgeSet = orEdgeSet(beforeEdgeSet, afterEdgeSet);
      	size = oEdgeSet.size();
      	return size;
    }
	
    
    
    /**OrderPairSetに対応している2つの集合から∇Setを返します。
    *@return ∇の集合
    */  
        private OrderPairSet nablaOrderPairSet(OrderPairSet teacherSet, OrderPairSet studentSet){
    	OrderPairSet sameEdgeSet = new OrderPairSet();
    	Iterator it = teacherSet.iterator();
    	while(it.hasNext()){
    	    Edge op = (Edge)it.next();
    	    if(studentSet.contains(op) == true){
    		sameEdgeSet.add(op);
    	    }
    	    studentSet.add(op);
    	}    	
    	Iterator it2 = sameEdgeSet.iterator();
    	while(it2.hasNext()){
    	    Edge ed = (Edge)it2.next();
    	    studentSet.remove(ed);
    	}
    	return studentSet;
        }
          
    /**OrderPairSetに対応している2つの集合から∇の大きさを返します。
    *@return ∇集合のサイズ
    */ 
        private int nablaOrderPairSetSize(OrderPairSet teacherEdgeSet, OrderPairSet studentEdgeSet){
          	int size = 0;
          	OrderPairSet lEdgeSet = new OrderPairSet();
          	lEdgeSet = nablaOrderPairSet(teacherEdgeSet, studentEdgeSet);
          	size = lEdgeSet.size();
          	return size;
        }
    /**OrderPairSetに対応している２つの集合からorSetを返します。
    *@return orの集合
    */       
        private OrderPairSet orOrderPairSet(OrderPairSet beforeEdgeSet, OrderPairSet afterEdgeSet){
          	Iterator ite = beforeEdgeSet.iterator();
          	while(ite.hasNext()){
    	    Edge ed = (Edge)ite.next();
    	    afterEdgeSet.add(ed);
          	}
          	return afterEdgeSet;
        }
          
    /**OrderPairSetに対応している２つの集合から論理和の大きさを返します。 
    *@return or集合のサイズ
    */       
        private int orOrderPairSetSize(OrderPairSet beforeEdgeSet, OrderPairSet afterEdgeSet){
          	int size = 0;
          	OrderPairSet oEdgeSet = new OrderPairSet();
          	oEdgeSet = orOrderPairSet(beforeEdgeSet, afterEdgeSet);
          	size = oEdgeSet.size();
          	return size;
        }  
    
    
    
/**
* 類似度を返します。
* @return 類似度
*/
    public double getSimilarityIndex(){
	double denominator = 0.0; //分母(OR)
	double numerator = 0.0; //分子(AND)
	
	//要素対を求めるマトリクス
	BooleanMatrix pairTeacherMatrix = new BooleanMatrix(getVertexCount(), getVertexCount());
	BooleanMatrix pairStudentMatrix = new BooleanMatrix(getVertexCount(), getVertexCount());
	
	//可到達マトリクス
	BooleanMatrix reachTeacherMatrix = new BooleanMatrix();
	reachTeacherMatrix.matrixCopy(getReachableMatrixOfTeacher());
	BooleanMatrix reachStudentMatrix = new BooleanMatrix();
	reachStudentMatrix.matrixCopy(getReachableMatrixOfStudent());
	
	//計算過程で使用するマトリクス
	BooleanMatrix resultMatrix = new BooleanMatrix(getVertexCount(), getVertexCount());
	for (int i = 0; i < getVertexCount(); i++){
	    for (int j = 0; j < getVertexCount(); j++){
		
		//教授者のマトリクスでその2要素間に枝があればその要素間の要素対集合を求める
		if (teacherMatrix.getCell(i, j) == true){
		    pairTeacherMatrix.matrixCopy(getPairMatrix(reachTeacherMatrix, i, j));
		}
		
		//枝がなければ要素対集合は空集合
		else{
		    pairTeacherMatrix.clearMatrix();
		}
		
		//学習者のマトリクスでその2要素間に枝があればその要素間の要素対集合を求める
		if (studentMatrix.getCell(i, j) == true){
		    pairStudentMatrix.matrixCopy(getPairMatrix(reachStudentMatrix, i, j));
		}
		
		//枝がなければ要素対集合は空集合
		else{
		    pairStudentMatrix.clearMatrix();
		}
		
		//分母の計算
		resultMatrix.matrixCopy(pairTeacherMatrix.getOrMatrix(pairStudentMatrix));
		denominator = resultMatrix.getElement() + denominator;
		
		//分子の計算
		resultMatrix.matrixCopy(pairTeacherMatrix.getAndMatrix(pairStudentMatrix));
		numerator = resultMatrix.getElement() + numerator;
	    }
	}
	
	return numerator / denominator;
    }
	
/**
* 到達度を返します。
* @return 到達度
*/
    public double getAttainmentIndex(){
	return Math.sqrt(getSimilarityIndex()) * 100;
    }
    
/**
* ある2点間における教授者と学習者の枝の状態を返します。
* どちらも枝が無ければ0、どちらも枝があれば1、教授者のマトリクスのみに枝がある場合は3、
* 学習者のマトリクスのみに枝がある場合は4が入ります。
* @param baseMatrix 比較元のマトリクス
* @param compareMatrix 比較されるマトリクス
* @param Ij 有向枝の始点
* @param Ik 有向枝の終点
* @return 枝の状態
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
* @exception DisagreementMatrixSizeException 2つのマトリクスサイズが一致しない場合
* @exception NotElementException 指定した要素番号が存在しない場合
* @see #getBlendMatrix()
*/
    private int getBlendEdge(BooleanMatrix baseMatrix, BooleanMatrix compareMatrix, int Ij, int Ik){
	if(baseMatrix.isSquare() && compareMatrix.isSquare()){
	    if (baseMatrix.isSameMatrixSize(compareMatrix)){
		int value;
		if (baseMatrix.isMatrixEntityPoint(Ij, Ik) && compareMatrix.isMatrixEntityPoint(Ij, Ik)){
		    if (baseMatrix.getCell(Ij, Ik) == compareMatrix.getCell(Ij, Ik)){
			if(baseMatrix.getCell(Ij, Ik) == LFT_EDGE_TRUE){
			    value = LFT_EDGE_INT_TRUE;
			}
			else{
			    value = LFT_EDGE_INT_FALSE;
			}
		    }
		    else if (baseMatrix.getCell(Ij, Ik) == LFT_EDGE_TRUE &&
			     compareMatrix.getCell(Ij, Ik) == LFT_EDGE_FALSE){
			value = LFT_EDGE_TEACHER;
		    }
		    else{
			value = LFT_EDGE_STUDENT;
		    }
		    return value;
		}
		else{
		    throw new NotElementException("point(" + Ij + "," + Ik + ")");
		}
	    }
	    else{
		throw new DisagreementMatrixSizeException();
	    }
	}
	else{
	    throw new NotSquareMatrixException();
	}
    }
	
/**
* ある2点間における教授者と学習者の枝の状態を返します。
* どちらも枝が無ければ0、どちらも枝があれば1、教授者のマトリクスのみに枝がある場合はT、
* 学習者のマトリクスのみに枝がある場合はSが入ります。
* @param matrix1 比較元のマトリクス
* @param matrix2 比較されるマトリクス
* @param Ij 有向枝の始点
* @param Ik 有向枝の終点
* @return 枝の状態
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
* @exception DisagreementMatrixSizeException 2つのマトリクスサイズが一致しない場合
* @exception NotElementException 指定した要素番号が存在しない場合
* @see #getBlendStringMatrix()
* @see #getBlendStringMatrix(BooleanMatrix baseMatrix, BooleanMatrix compareMatrix)
*/
    public String getStateOfEdge(BooleanMatrix matrix1, BooleanMatrix matrix2, int Ij, int Ik){
	if(matrix1.isSquare() && matrix2.isSquare()){
	    if (matrix1.isSameMatrixSize(matrix2)){
		String st;
		if (matrix1.isMatrixEntityPoint(Ij, Ik) && matrix2.isMatrixEntityPoint(Ij, Ik)){
		    if (matrix1.getCell(Ij,Ik) == matrix2.getCell(Ij, Ik)){
			if(matrix1.getCell(Ij,Ik) == LFT_EDGE_TRUE){
			    st = LFT_EDGE_STRING_TRUE;
			}
			else{
			    st = LFT_EDGE_STRING_FALSE;
			}
		    }
		    else if (matrix1.getCell(Ij,Ik) == LFT_EDGE_TRUE &&
			     matrix2.getCell(Ij,Ik) == LFT_EDGE_FALSE){
			st = LFT_EDGE_STRING_TEACHER;
		    }
		    else{
			st = LFT_EDGE_STRING_STUDENT;
		    }
		    return st;
		}
		else{
		    throw new NotElementException("point(" + Ij + "," + Ik + ")");
		}
	    }
	    else{
		throw new DisagreementMatrixSizeException();
	    }
	}
	else{
	    throw new NotSquareMatrixException();
	}
    }
	
    
    /**
    * ある2点間における教授者と学習者の枝の状態を返します。
    * どちらも枝が無ければ0、どちらも枝があれば1、教授者のマトリクスのみに枝がある場合はT、
    * 学習者のマトリクスのみに枝がある場合はSが入ります。
    * @param matrix1 比較元のマトリクス
    * @param matrix2 比較されるマトリクス
    * @param v1 有向枝の始点
    * @param v2 有向枝の終点
    * @return 枝の状態
    * @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
    * @exception DisagreementMatrixSizeException 2つのマトリクスサイズが一致しない場合
    * @exception NotElementException 指定した要素番号が存在しない場合
    * @see #getBlendStringMatrix()
    * @see #getBlendStringMatrix(BooleanMatrix baseMatrix, BooleanMatrix compareMatrix)
    */
        public String getStateOfEdge(BooleanMatrix matrix1, BooleanMatrix matrix2, Vertex v1, Vertex v2){
    int Ij = v1.getVertex(v1);
    int Ik = v2.getVertex(v2);
        	if(matrix1.isSquare() && matrix2.isSquare()){
    	    if (matrix1.isSameMatrixSize(matrix2)){
    		String st;
    		if (matrix1.isMatrixEntityPoint(Ij, Ik) && matrix2.isMatrixEntityPoint(Ij, Ik)){
    		    if (matrix1.getCell(Ij,Ik) == matrix2.getCell(Ij, Ik)){
    			if(matrix1.getCell(Ij,Ik) == LFT_EDGE_TRUE){
    			    st = LFT_EDGE_STRING_TRUE;
    			}
    			else{
    			    st = LFT_EDGE_STRING_FALSE;
    			}
    		    }
    		    else if (matrix1.getCell(Ij,Ik) == LFT_EDGE_TRUE &&
    			     matrix2.getCell(Ij,Ik) == LFT_EDGE_FALSE){
    			st = LFT_EDGE_STRING_TEACHER;
    		    }
    		    else{
    			st = LFT_EDGE_STRING_STUDENT;
    		    }
    		    return st;
    		}
    		else{
    		    throw new NotElementException("point(" + Ij + "," + Ik + ")");
    		}
    	    }
    	    else{
    		throw new DisagreementMatrixSizeException();
    	    }
    	}
    	else{
    	    throw new NotSquareMatrixException();
    	}
        }
    
/**
* 指定した2つの隣接マトリクスを一緒にしたマトリクスを返します。<BR>
* どちらも枝が無ければ0、どちらも枝があれば1、baseMatrixで指定したマトリクスのみに枝がある場合は3、
* compareMatrixで指定したマトリクスにのみに枝がある場合は4が入ります。
* 一緒にしたマトリクスの行名と列名はbaseMatrixで指定したマトリクスと同じになります。
* @param baseMatrix 比較元のマトリクス
* @param compareMatrix 比較されるマトリクス
* @return 一緒にしたマトリクス
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
* @exception DisagreementMatrixSizeException 2つのマトリクスサイズが一致しない場合
*/
    private ArithmeticMatrix getBlendMatrix(BooleanMatrix baseMatrix, BooleanMatrix compareMatrix){
	if(baseMatrix.isSquare() && compareMatrix.isSquare()){
	    if (baseMatrix.isSameMatrixSize(compareMatrix)){
		ArithmeticMatrix returnMatrix = new ArithmeticMatrix(getVertexCount(), getVertexCount(),
								     baseMatrix.getRowNameArray(), baseMatrix.getColNameArray());
		
		for (int i = 0; i < getVertexCount(); i++){
		    for (int j = 0; j < getVertexCount(); j++){
			returnMatrix.setCell(i, j, getBlendEdge(baseMatrix, compareMatrix, i ,j));
		    }
		}
		return returnMatrix;
	    }
	    else{
		throw new DisagreementMatrixSizeException();
	    }
	}
	else{
	    throw new NotSquareMatrixException();
	}
    }
	
/**
* 指定した2つの隣接マトリクスを一緒にしたマトリクスを返します。<BR>
* どちらも枝が無ければ0、どちらも枝があれば1、baseMatrixで指定したマトリクスのみに枝がある場合はT、
* compareMatrixで指定したマトリクスにのみに枝がある場合はSが入ります。
* 一緒にしたマトリクスの行名と列名はbaseMatrixで指定したマトリクスと同じになります。
* @param matrix1 比較元のマトリクス
* @param matrix2 比較されるマトリクス
* @return 一緒にしたマトリクス
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
* @exception DisagreementMatrixSizeException 2つのマトリクスサイズが一致しない場合
* @see #getBlend(BooleanMatrix baseMatrix, BooleanMatrix compareMatrix, int Ij, int Ik)
* @see #getBlendStringMatrix()
*/
    public StringMatrix getStateOfEdgeStringMatrix(BooleanMatrix matrix1, BooleanMatrix matrix2){
	if(matrix1.isSquare() && matrix2.isSquare()){
	    if (matrix1.isSameMatrixSize(matrix2)){
		StringMatrix returnMatrix = new StringMatrix(getVertexCount(), getVertexCount(),
							     matrix1.getRowNameArray(), matrix1.getColNameArray());
		
		for (int i = 0; i < getVertexCount(); i++){
		    for (int j = 0; j < getVertexCount(); j++){
			returnMatrix.setCell(i, j, getStateOfEdge(matrix1, matrix2, i, j));
		    }
		}
		return returnMatrix;
	    }
	    else{
		throw new DisagreementMatrixSizeException();
	    }
	}
	else{
	    throw new NotSquareMatrixException();
	}
    }
	
/**
* 教授者のマトリクスと学習者のマトリクスを一緒にしたマトリクスを返します。<BR>
* どちらも枝が無ければ0、どちらも枝があれば1、教授者のマトリクスのみに枝がある場合は3、
* 学習者のマトリクスのみに枝がある場合は4が入ります。
* 一緒にしたマトリクスの行名と列名は教授者のマトリクスと同じになります。
* @return 一緒にしたマトリクス
*/
    private ArithmeticMatrix getBlendMatrix(){
	return getBlendMatrix(teacherMatrix, studentMatrix);
    }
	
/**
* 教授者のマトリクスと学習者のマトリクスを一緒にしたマトリクスを返します。<BR>
* どちらも枝が無ければ0、どちらも枝があれば1、教授者のマトリクスのみに枝がある場合はT、
* 学習者のマトリクスのみに枝がある場合はSが入ります。
* 一緒にしたマトリクスの行名と列名は教授者のマトリクスと同じになります。
* @return 一緒にしたマトリクス
* @see #getBlend(BooleanMatrix baseMatrix, BooleanMatrix compareMatrix, int Ij, int Ik)
* @see #getBlendStringMatrix(BooleanMatrix baseMatrix, BooleanMatrix compareMatrix)
*/
    public StringMatrix getStateOfEdgeStringMatrix(){
	return getStateOfEdgeStringMatrix(teacherMatrix, studentMatrix);
    }
	
/**
* 隣接マトリクスを元にある要素間の要素対集合を求めます。
* @param matrix 調べる隣接マトリクス
* @param Ij 有向枝の始点
* @param Ik 有向枝の先の終点
* @return どの要素間が要素対か示したマトリクス
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
* @exception NotElementException 指定した要素番号が存在しない場合
*/
    private BooleanMatrix getVertexPairMatrix(BooleanMatrix matrix, int Ij, int Ik){
	if(matrix.isSquare()){
	    if (matrix.isMatrixEntityPoint(Ij, Ik)){
		
		//可到達マトリクスを生成
		BooleanMatrix lookMatrix = new BooleanMatrix(matrix.getRowLength(), matrix.getColLength(),
							     matrix.getRowNameArray(), matrix.getColNameArray());
		lookMatrix.matrixCopy(matrix.getReachMatrix());
		
		BooleanMatrix returnMatrix = new BooleanMatrix(lookMatrix.getRowLength(), lookMatrix.getColLength(), 
							       lookMatrix.getRowNameArray(), lookMatrix.getColNameArray());
		
		//枝が存在するかどうか
		if (matrix.getCell(Ij, Ik) == LFT_EDGE_TRUE){
		    returnMatrix.matrixCopy(getPairMatrix(lookMatrix, Ij, Ik));
		}
		return returnMatrix;
	    }
	    else{
		throw new NotElementException("point(" + Ij + "," + Ik + ")");
	    }
	}
	else{
	    throw new NotSquareMatrixException();
	}
    }
	
/**
* 隣接マトリクスを元にある要素間の要素対集合を求めます。
* @param matrix 調べる隣接マトリクス
* @param Ij 有向枝の始点
* @param Ik 有向枝の先の終点
* @return 要素対集合となる枝の集合
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
* @exception NotElementException 指定した要素番号が存在しない場合
*/
    public OrderPairSet getOrderPairSet(BooleanMatrix matrix,Edge ed){
    	OrderPairSet o = new OrderPairSet();
    	return o;
    }
    
    private EdgeSet getVertexPairSet(BooleanMatrix matrix, int Ij, int Ik){
	if(matrix.isSquare()){
	    if (matrix.isMatrixEntityPoint(Ij, Ik)){
		
		//可到達マトリクスを生成
		BooleanMatrix lookMatrix = new BooleanMatrix(matrix.getRowLength(), matrix.getColLength(),
							     matrix.getRowNameArray(), matrix.getColNameArray());
		lookMatrix.matrixCopy(matrix.getReachMatrix());
		
		EdgeSet edgeHash = new EdgeSet();
		
		//枝が存在するかどうか
		if (matrix.getCell(Ij, Ik) == LFT_EDGE_TRUE){
		    for (int i = 0; i < lookMatrix.getRowLength(); i++){
			for (int j = 0; j < lookMatrix.getColLength(); j++){
			    if (lookMatrix.getCell(i, Ij) && lookMatrix.getCell(Ik, j) == LFT_EDGE_TRUE){
				edgeHash.add(new Edge(i,j));
			    }
			}
		    }
		}
		return edgeHash;
	    }
	    else{
		throw new NotElementException("point(" + Ij + "," + Ik + ")");
	    }
	}
	else{
	    throw new NotSquareMatrixException();
	}
    }
	
/**
* 可到達マトリクスを元にある要素間の要素対集合を求めます。<BR>
* 隣接マトリクスから求めるのに比べ、2要素間が隣接しているかが保証されないため、
* このメソッドを使用する場合はその要素間同士が隣接していることをチェックする必要があります。
* @param lookMatrix 調べる可到達マトリクス
* @param Ij 有向枝の元の要素番号
* @param Ik 有向枝の先の要素番号
* @return どの要素間が要素対か示したマトリクス
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
* @exception NotElementException 指定した要素番号が存在しない場合
*/
    private BooleanMatrix getPairMatrix(BooleanMatrix lookMatrix, int Ij, int Ik){
	if(lookMatrix.isSquare()){
	    if (lookMatrix.isMatrixEntityPoint(Ij, Ik)){
		BooleanMatrix returnMatrix = new BooleanMatrix(lookMatrix.getRowLength(),lookMatrix.getColLength(), 
							       lookMatrix.getRowNameArray(), lookMatrix.getColNameArray());
		for (int i = 0; i < lookMatrix.getRowLength(); i++){
		    for (int j = 0; j < lookMatrix.getColLength(); j++){
			if (lookMatrix.getCell(i, Ij) && lookMatrix.getCell(Ik, j) == LFT_EDGE_TRUE){
			    returnMatrix.setCell(i, j, LFT_EDGE_TRUE);
			}
			else{
			    returnMatrix.setCell(i, j, LFT_EDGE_FALSE);
			}
		    }
		}
		return returnMatrix;
	    }
	    else{
		throw new NotElementException("point(" + Ij + "," + Ik + ")");
	    }
	}
	else{
	    throw new NotSquareMatrixException();
	}
    }
	
/**
* マトリクスに存在する枝の重要度を求めます。
* @param matrix 重要度を求める対象の隣接マトリクス
* @return 枝の重要度を格納したマトリクス
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
*/
    public ArithmeticMatrix getImportantEdgeMatrix(BooleanMatrix matrix){
	if(matrix.isSquare()){
	    ArithmeticMatrix returnMatrix = new ArithmeticMatrix(matrix.getRowLength(), matrix.getColLength(),
								 matrix.getRowNameArray(), matrix.getColNameArray());
	    
	    for (int i = 0;i < matrix.getRowLength(); i++){
		for (int j = 0; j < matrix.getColLength(); j++){
		    returnMatrix.setCell(i, j, getImportantIndex(matrix, i, j));
		}
	    }
	    return returnMatrix;
	}
	else{
	    throw new NotSquareMatrixException();
	}
    }
	
/**
* マトリクスの2項目間の枝の重要度を返します。
* @param matrix 重要度を求めるマトリクス
* @param Ij 有向枝の始点となる項目Ij
* @param Ik 有向枝の終点となる項目Ik
* @return 指定した項目間の重要度
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
*/
    public double getImportantIndex(BooleanMatrix matrix, int Ij, int Ik){
	if(matrix.isSquare()){
	    BooleanMatrix temporaryMatrix = new BooleanMatrix(matrix.getRowLength(), matrix.getColLength());
	    double cont = Math.floor(getVertexCount() / 2) * Math.floor((getVertexCount() + 1) / 2);
	    double value = 0.0;
	    
	    //枝が存在するかどうか
	    if (matrix.getCell(Ij, Ik) == LFT_EDGE_TRUE){
		temporaryMatrix.matrixCopy(getVertexPairMatrix(matrix, Ij, Ik));
		value = temporaryMatrix.getElement();
		value = value / cont;
	    }
	    return value;
	}
	else{
	    throw new NotSquareMatrixException();
	}
    }
	
    /**
    * マトリクスの2項目間の枝の重要度を返します。
    * @param matrix 重要度を求めるマトリクス
    * @param ed 有向枝
    * @return 指定した項目間の重要度
    * @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
    */
        public double getImportantIndex(BooleanMatrix matrix, Edge ed){
        	int Ij = ed.getBegin();
        	int Ik = ed.getEnd();
    	if(matrix.isSquare()){
    	    BooleanMatrix temporaryMatrix = new BooleanMatrix(matrix.getRowLength(), matrix.getColLength());
    	    double cont = Math.floor(getVertexCount() / 2) * Math.floor((getVertexCount() + 1) / 2);
    	    double value = 0.0;
    	    
    	    //枝が存在するかどうか
    	    if (matrix.getCell(Ij, Ik) == LFT_EDGE_TRUE){
    		temporaryMatrix.matrixCopy(getVertexPairMatrix(matrix, Ij, Ik));
    		value = temporaryMatrix.getElement();
    		value = value / cont;
    	    }
    	    return value;
    	}
    	else{
    	    throw new NotSquareMatrixException();
    	}
        }
    
    
/**
* 教授者のマトリクスにある指定した枝の重要度を返します。
* @param Ij 有向枝の始点
* @param Ik 有向枝の終点
* @return 重要度
* @exception NotElementException 指定した要素番号が存在しない場合
* @see #getImportantEdgeStudent(int Ij, int Ik)
* @see #getEdgeTeacherMatrix()
*/
    public double getImportantEdgeTeacher(int Ij, int Ik){
	if (teacherMatrix.isMatrixEntityPoint(Ij, Ik)){
	    return getImportantIndex(teacherMatrix, Ij, Ik);
	}
	else{
	    throw new NotElementException("point(" + Ij + "," + Ik + ")");
	}
    }
	
    /**
    * 教授者のマトリクスにある指定した枝の重要度を返します。
    * @param ed 有向枝
    * @return 重要度
    * @exception NotElementException 指定した要素番号が存在しない場合
    * @see #getImportantEdgeStudent(int Ij, int Ik)
    * @see #getEdgeTeacherMatrix()
    */
        public double getImportantEdgeTeacher(Edge ed){
        	int Ij = ed.getBegin();
        	int Ik = ed.getEnd();
    	if (teacherMatrix.isMatrixEntityPoint(Ij, Ik)){
    	    return getImportantIndex(teacherMatrix, Ij, Ik);
    	}
    	else{
    	    throw new NotElementException("point(" + Ij + "," + Ik + ")");
    	}
        }  
    
/**
* 教授者の枝の重要度を求めたマトリクスを返します。
* @return 重要度を求めたマトリクス
* @see #getEdgeStudentMatrix()
* @see #getImportantEdgeTeacher(int Ij, int Ik)
*/
    public ArithmeticMatrix getImportantIndexOfTeacher(){
	return getImportantEdgeMatrix(teacherMatrix);
    }
	
/**
* 学習者のマトリクスにある指定した枝の重要度を返します。
* @param Ij 有向枝の始点
* @param Ik 有向枝の終点
* @return 重要度
* @exception NotElementException 指定した要素番号が存在しない場合
* @see #getEdgeTeacherMatrix()
* @see #getImportantEdgeStudent(int Ij, int Ik)
*/
    public double getImportantEdgeStudent(int Ij, int Ik){
	if (studentMatrix.isMatrixEntityPoint(Ij, Ik)){
	    return getImportantIndex(studentMatrix, Ij, Ik);
	}
	else{
	    throw new NotElementException("point(" + Ij + "," + Ik + ")");
	}
    }
    
    /**
    * 学習者のマトリクスにある指定した枝の重要度を返します。
    * @param ed 有向枝
    * @return 重要度
    * @exception NotElementException 指定した要素番号が存在しない場合
    * @see #getEdgeTeacherMatrix()
    * @see #getImportantEdgeStudent(int Ij, int Ik)
    */
        public double getImportantEdgeStudent(Edge ed){
        	int Ij = ed.getBegin();
        	int Ik = ed.getEnd();
    	if (studentMatrix.isMatrixEntityPoint(Ij, Ik)){
    	    return getImportantIndex(studentMatrix, Ij, Ik);
    	}
    	else{
    	    throw new NotElementException("point(" + Ij + "," + Ik + ")");
    	}
        }   
	
/**
* 学習者の枝の重要度を求めたマトリクスを返します。
* @return 重要度を求めたマトリクス
* @see #getEdgeStudentMatrix()
* @see #getImportantEdgeTeacher(int Ij, int Ik)
*/
    public ArithmeticMatrix getImportantIndexOfStudent(){
	return getImportantEdgeMatrix(studentMatrix);
    }
	
/**
* 教授者の隣接マトリクスを返します。
* @return 教授者のマトリクス
* @see #getAdjacencyMatrixOfStudent()
*/
    public BooleanMatrix getAdjancencyMatrixOfTeacher(){
	return teacherMatrix;
    }
	
/**
* 学習者の隣接マトリクスを返します。
* @return 学習者のマトリクス
* @see #getAdjacencyMatrixOfTeacher()
*/
    public BooleanMatrix getAdjacencyMatrixOfStudent(){
	return studentMatrix;
    }
	
/**
* 教授者の可到達マトリクスを返します。
* @return 教授者の可到達マトリクス
* @see #getReachableMatrixOfStudent()
*/
    public BooleanMatrix getReachableMatrixOfTeacher(){
	BooleanMatrix reachTeacherMatrix = new BooleanMatrix();
	reachTeacherMatrix.matrixCopy(teacherMatrix.getReachMatrix());
	return reachTeacherMatrix;
    }
	
/**
* 学習者の可到達マトリクスを返します。
* @return 学習者の可到達マトリクス
* @see #getReachableMatrixOfTeacher()
*/
    public BooleanMatrix getReachableMatrixOfStudent(){
	BooleanMatrix reachStudentMatrix = new BooleanMatrix();
	reachStudentMatrix.matrixCopy(studentMatrix.getReachMatrix());
	return reachStudentMatrix;
    }
	
/**
* 教授者のマトリクスにおける要素数(行または列数)を返します。
* @return 要素数
*/
    public int getVertexCount(){
	return teacherMatrix.getRowLength();
    }
	
/**
* 教授者のマトリクスにある有向枝の数を返します。
* @return 教授者のマトリクスにある有向枝の数
* @see #getStudentEdge()
* @see #getTeacherOnlyEdge()
* @see #getTotelEdge()
* @see #getAccordEdge()
*/
    public int getEdgeCountOfTeacher(){
	return teacherMatrix.getElement();
    }
	
/**
* 学習者のマトリクスにある有向枝の数を返します。
* @return 学習者のマトリクスにある有向枝の数
* @see #getTeacherEdge()
* @see #getStudentOnlyEdge()
* @see #getTotelEdge()
* @see #getAccordEdge()
*/
    public int getEdgeCountOfStudent(){
	return studentMatrix.getElement();
    }
	
/**
* 教授者のマトリクスと学習者のマトリクスのどちらか一方にでも存在する有向枝の数を返します。
* @return どちらか一方にでもある有向枝の数
* @see #getAccordEdge()
* @see #getTeacherEdge()
* @see #getStudentEdge()
* @see #getTeacherOnlyEdge()
* @see #getStudentOnlyEdge()
*/
    public int getEitherOneEdgeCount(){
	ArithmeticMatrix mixMatrix = new ArithmeticMatrix(getVertexCount(), getVertexCount());
	mixMatrix.matrixCopy(getBlendMatrix());
	return mixMatrix.getElement();
    }
	
/**
* 教授者のマトリクスにのみ存在する有向枝の数を返します。
* @return 教授者のマトリクスにのみ存在する有向枝の数
* @see #getTeacherEdge()
* @see #getStudentOnlyEdge()
* @see #getTotelEdge()
* @see #getAccordEdge()
*/
    public int getOnlyEdgeCountOfTeacher(){
	int count = 0;
	ArithmeticMatrix mixMatrix = new ArithmeticMatrix(getVertexCount(), getVertexCount());
	mixMatrix.matrixCopy(getBlendMatrix());
	for (int i = 0; i < mixMatrix.getRowLength(); i++){
	    for (int j = 0; j < mixMatrix.getColLength(); j++){
		if (mixMatrix.getCell(i, j) == LFT_EDGE_TEACHER){
		    count = count + 1;
		}
	    }
	}
	return count;
    }
	
/**
* 学習者のマトリクスにのみ存在する有向枝の数を返します。
* @return 学習者のマトリクスにのみ存在する有向枝の数
* @see #getStudentEdge()
* @see #getTeacherOnlyEdge()
* @see #getTotelEdge()
* @see #getAccordEdge()
*/
    public int getOnlyEdgeCountOfStudent(){
	int count = 0;
	ArithmeticMatrix mixMatrix = new ArithmeticMatrix(getVertexCount(), getVertexCount());
	mixMatrix.matrixCopy(getBlendMatrix());
	for (int i = 0; i < mixMatrix.getRowLength(); i++){
	    for (int j = 0; j < mixMatrix.getColLength(); j++){
		if (mixMatrix.getCell(i, j) == LFT_EDGE_STUDENT){
		    count = count + 1;
		}
	    }
	}
	return count;
    }
	
/**
* 教授者のマトリクスにも学習者のマトリクスにも存在する有向枝の数を返します。
* @return 両者にある有向枝の数
* @see #getTotelEdge()
* @see #getTeacherEdge()
* @see #getStudentEdge()
* @see #getTeacherOnlyEdge()
* @see #getStudentOnlyEdge()
*/
    public int getAccordEdgeCount(){
	int count = 0;
	ArithmeticMatrix mixMatrix = new ArithmeticMatrix(getVertexCount(), getVertexCount());
	mixMatrix.matrixCopy(getBlendMatrix());
	for (int i = 0; i < mixMatrix.getRowLength(); i++){
	    for (int j = 0; j < mixMatrix.getColLength(); j++){
		if (mixMatrix.getCell(i, j) == LFT_EDGE_INT_TRUE){
		    count = count + 1;
		}
	    }
	}
	return count;
    }
	
/**
* 教授者のマトリクスを設定します。
* @param baseTeacherMatrix 設定をする教授者のマトリクス
* @exception NotSquareMatrixException 教授者のマトリクスが正方行列ではない場合
*/
    private void setTeacherMatrix(BooleanMatrix baseTeacherMatrix){
	if (baseTeacherMatrix.isSquare()){
	    teacherMatrix = new BooleanMatrix();
	    teacherMatrix.matrixCopy(baseTeacherMatrix);
	}
	else{
	    throw new NotSquareMatrixException();
	}
    }
	
/**
* 学習者のマトリクスを設定します。<BR>
* 学習者のマトリクスは正方行列であり、教授者のマトリクスとサイズが一致する必要があります。
* @param baseStudentMatrix 設定をする学習者のマトリクス
* @exception NotSquareMatrixException 学習者のマトリクスが正方行列ではない場合
* @exception DisagreementMatrixSizeException 教授者のマトリクスと学習者のマトリクスサイズが一致しない場合
*/
    public void setMatrixOfStudent(BooleanMatrix StudentMatrix){
	if (StudentMatrix.isSquare()){
	    if (teacherMatrix.isSameMatrixSize(StudentMatrix)){
		studentMatrix = new BooleanMatrix();
		studentMatrix.matrixCopy(StudentMatrix);
	    }
	    else{
		throw new DisagreementMatrixSizeException();
	    }
	}
	else{
	    throw new NotSquareMatrixException();
	}
    }
	
}
