package jp.ac.takushoku_u.cs;

/**
* SSGraphPointクラスはSSグラフを描くための座標情報を求めるためのクラスです。
* @version 1.0
* @author 笠島 裕史(Kasajima Hiroshi)
*/
/*
* 最終更新日 2005年03月01日
*/
public class SSGraphPoint{
	
/**
* 隣接マトリクス
*/
	private BooleanMatrix matrix;
	
/**
* 可到達マトリクス
*/
	private BooleanMatrix reachMatrix;
	
/**
* 最大レベル(最大平均評定値)
*/
	private double maxLevel;
	
/**
* 描画要素数
*/
	private int elementNumber;
	
/**
* 配置レベル配列(配列数は要素数)
*/
	private int[] paintLevel;
	
/**
* 縦軸値配列
*/
	private double[] heigthNumber;
	
/**
* 描画順配列
*/
	private int[] paintNumber;
	
/**
* 横幅配列(配列数はレベル数)
*/
	private int[] paintWide;
	
/**
* 同一レベルでの描画位置
*/
	private int[] paintWideNumber;
	
/**
* 各要素のx,y座標
*/
	private int[][] paintPoint;
	
/**
* 描画する要素の大きさ
*/
	private int elementSize = 20;
	
/**
* 描画するときの1ブロックの大きさ
*/
	private int blockSize = 50;
	
/**
* フレームのサイズ(y軸)
*/
	private int flameSize = 700;
	
	
/**
* 要素のないスケルトンマトリクスを設定します。
*/
	public SSGraphPoint(){
		BooleanMatrix mat = new BooleanMatrix();
		setMatrix(mat);
	}
	
/**
* スケルトンマトリクスを設定し、描画位置情報を求めます。
* @param skeletonMatrix スケルトンマトリクス
* @param maxValue 評定の最大値
* @param heightArray グラフを描く順に並べ替えた配列
* @param averageArray グラフの縦軸(平均評定値)
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
* @see jp.ac.takushoku_u.cs.SSAnalysis#getSkeletonMatrix()
* @see jp.ac.takushoku_u.cs.SSAnalysis#getMaxValue()
* @see jp.ac.takushoku_u.cs.SSAnalysis#getGraphElementArray()
* @see jp.ac.takushoku_u.cs.SSAnalysis#getGraphHeightArray()
*/
	public SSGraphPoint(BooleanMatrix skeletonMatrix, double maxValue, int[] heightArray, double[] averageArray){
		setMatrix(skeletonMatrix);
		setAverageArray(averageArray);
		setHeightArray(heightArray);
		setMaxValue(maxValue);
	}
	
/**
* スケルトンマトリクスを設定し、描画位置情報を求めます。
* @param skeletonMatrix スケルトンマトリクス
* @param maxValue 評定の最大値
* @param heightArray グラフを描く順に並べ替えた配列
* @param averageArray グラフの縦軸(平均評定値)
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
* @see jp.ac.takushoku_u.cs.SSAnalysis#getSkeletonMatrix()
* @see jp.ac.takushoku_u.cs.SSAnalysis#getMaxValue()
* @see jp.ac.takushoku_u.cs.SSAnalysis#getGraphElementArray()
* @see jp.ac.takushoku_u.cs.SSAnalysis#getGraphHeightArray()
*/
	public SSGraphPoint(ArithmeticMatrix skeletonMatrix, double maxValue, int[] heightArray, double[] averageArray){
		setMatrix(skeletonMatrix.toBooleanMatrix());
		setAverageArray(averageArray);
		setHeightArray(heightArray);
		setMaxValue(maxValue);
	}
	
/**
* スケルトンマトリクスを設定し、描画位置情報を求めます。
* @param skeletonMatrix スケルトンマトリクス
* @param maxValue 評定の最大値
* @param averageArray グラフの縦軸(平均評定値)
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
* @see jp.ac.takushoku_u.cs.SSAnalysis#getSkeletonMatrix()
* @see jp.ac.takushoku_u.cs.SSAnalysis#getMaxValue()
* @see jp.ac.takushoku_u.cs.SSAnalysis#getGraphElementArray()
* @see jp.ac.takushoku_u.cs.SSAnalysis#getGraphHeightArray()
*/
	public SSGraphPoint(BooleanMatrix skeletonMatrix, double maxValue, double[] averageArray){
		setMatrix(skeletonMatrix);
		setAverageArray(averageArray);
		setHeightArray(getGraphArray(averageArray));
		setMaxValue(maxValue);
	}
	
/**
* スケルトンマトリクスを設定し、描画位置情報を求めます。
* @param skeletonMatrix スケルトンマトリクス
* @param maxValue 評定の最大値
* @param averageArray グラフの縦軸(平均評定値)
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
* @see jp.ac.takushoku_u.cs.SSAnalysis#getSkeletonMatrix()
* @see jp.ac.takushoku_u.cs.SSAnalysis#getMaxValue()
* @see jp.ac.takushoku_u.cs.SSAnalysis#getGraphElementArray()
* @see jp.ac.takushoku_u.cs.SSAnalysis#getGraphHeightArray()
*/
	public SSGraphPoint(ArithmeticMatrix skeletonMatrix, double maxValue, double[] averageArray){
		setMatrix(skeletonMatrix.toBooleanMatrix());
		setAverageArray(averageArray);
		setHeightArray(getGraphArray(averageArray));
		setMaxValue(maxValue);
	}

	
/**
* グラフを描く際の各要素のレベルを配列として返します。
* @return レベルを格納した配列(配列の大きさは描画を行う項目数)
*/
	public int[] getPaintLevel(){
		int[] returnArray = new int[matrix.getRowLength()];
		BooleanMatrix reachMatrix= new BooleanMatrix();
		reachMatrix.matrixCopy(matrix.getReachMatrix());
		
		//配列の初期化
		for(int i = 0; i < returnArray.length; i++){
			returnArray[i] = -1;
		}
		
		boolean flag = true;
		
		//レベルを格納する
		int level = 0;
		int count = 0;
		for(int i = 0; i < returnArray.length; i++){
			int lookElement = paintNumber[i];
			returnArray[lookElement] = level;
			
			//次も同一レベルに描く訳でなければ、
			//その要素に到達する枝の本数を数える
			if(count == 0){
				for(int j = i + 1; j < returnArray.length; j++){
					if(matrix.getCell(paintNumber[j] , lookElement)){
						count++;
					}
				}
				level++;
				count--;
			}
			else if(i < returnArray.length-1 &&(!matrix.getCell(paintNumber[i+1],paintNumber[i]) ||
				(matrix.getCell(paintNumber[i+1],paintNumber[i])&&matrix.getCell(paintNumber[i],paintNumber[i+1])))){
				count--;
			}
		}
		
		//ルートの位置を変更する
		for(int i = 0; i < returnArray.length; i++){
			if(returnArray[i] == 0){
				int countA = 0;
				for(int j = 0; j < matrix.getRowLength(); j++){
					if(matrix.getCell(j,i)){
						if(countA < returnArray[j]){
							countA = returnArray[j];
						}
					}
				}
				returnArray[i] = countA - 1;
			}
		}
		
		return returnArray;
	}
	
/**
* どこに配置するか座標(x,y)を指定した配列を返します。
* @return 座標を指定した配列
*/
	public int[][] getPointArray(){
		paintLevel = new int[elementNumber];
		System.arraycopy(getPaintLevel(), 0, paintLevel, 0, elementNumber);
		getPaintWide();
		getPaintWideNumber();
		int maxWide = getColMaxLevel(paintLevel);
		
		int center = (maxWide+1) * blockSize / 2;
		
		//各レベルで既に描いた要素数をカウントします。
		int[] countArray = new int [getRowMaxLevel(getPaintLevel())+1];
		
		//センタリング
		for(int i = 0; i <= getRowMaxLevel(getPaintLevel()); i++){
			if(maxWide%2==0){
				countArray[i] = (maxWide + 1 - paintWide[i]) / 2;
			}
			else{
				countArray[i] = (maxWide - paintWide[i]) / 2;
			}
		}
		
		//y座標の間隔
			double interval;
			
			//平均評定値の間隔を求めます。
			if(getRowMaxLevel(getPaintLevel()) <= flameSize - blockSize){
				interval = (flameSize - blockSize*2) / maxLevel;
			}
			else{
				interval = maxLevel / (flameSize - blockSize*2);
			}
		
		
		//各要素のx座標とy座標を決めます
		for(int i = 0; i < elementNumber; i++){
			//横が偶数個
			if(paintWide[paintLevel[i]] % 2 == 0){
				if(paintWideNumber[i] > paintWide[paintLevel[i]] / 2){
					paintPoint[i][0] = (countArray[paintLevel[i]] + paintWideNumber[i] + 1) * blockSize;
				}
				else{
					paintPoint[i][0] = (countArray[paintLevel[i]] + paintWideNumber[i]) * blockSize;
				}
			}
			
			//横が奇数個
			else{
				paintPoint[i][0] = (countArray[paintLevel[i]] + paintWideNumber[i]) * blockSize;
			}
			
			
			//y座標を決定
			paintPoint[i][1] = (int)((flameSize - blockSize)- heigthNumber[i] * interval);
				
		}
		
		//y座標が接近する場合
		for(int i = 0; i < paintPoint.length; i++){
			for(int j = 0; j < paintPoint.length; j++){
				if(i != j && paintPoint[j][1] >= paintPoint[i][1] &&
				paintPoint[j][1] <= paintPoint[i][1] + elementSize && paintPoint[j][0] == paintPoint[i][0]){
					paintPoint[j][0] = paintPoint[i][0] + blockSize / 2;
					
				}
			}
		}
		
		return paintPoint;
	}
	
/**
* スケルトンマトリクスを設定します。
* @param skeletonMatrix スケルトンマトリクス
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
* @see jp.ac.takushoku_u.cs.SSAnalysis#getSkeletonMatrix()
*/
	public void setMatrix(BooleanMatrix skeletonMatrix){
		if(skeletonMatrix.isSquare()){
			matrix = new BooleanMatrix();
			reachMatrix = new BooleanMatrix();
			matrix.matrixCopy(skeletonMatrix);
			reachMatrix.matrixCopy(matrix.getReachMatrix());
			elementNumber = skeletonMatrix.getRowLength();
			paintPoint = new int[elementNumber][2];
		}
		else{
			throw new NotSquareMatrixException("row("+ skeletonMatrix.getRowLength() +
				"),col("+skeletonMatrix.getColLength()+")");
		}
	}
	
/**
* 最大評定値(グラフの縦軸の最大値)を設定します。
* 設定されている各項目の平均評定値よりも低い値が設定された場合、
* 最大の評定値となります。
* @param max 最大評定値
* @see jp.ac.takushoku_u.cs.SSAnalysis#getMaxValue()
*/
	public void setMaxValue(double max){
		maxLevel = max;
		double maxValue = max;
		for(int i = 0; i < heigthNumber.length; i++){
			if(heigthNumber[i] > max){
				maxValue = heigthNumber[i];
			}
		}
	}
	
/**
* 各項目の平均評定値(縦軸値)を設定します。
* @param averageArray 平均評定値
* @see jp.ac.takushoku_u.cs.SSAnalysis#getGraphHeightArray()
*/
	public void setAverageArray(double[] averageArray){
		if(matrix.getRowLength() == averageArray.length){
			heigthNumber = new double[averageArray.length];
			System.arraycopy(averageArray, 0, heigthNumber, 0, averageArray.length);
			
		}
		else{
			throw new DisagreementArraySizeException("matrix("+matrix.getRowLength()+
				"),array("+averageArray.length+")");
		}
	}
	
/**
* 項目をグラフを描く順に並べ替えた配列を設定します。
* @param heightArray グラフを描く順に項目を並べ替えた配列
* @exception DisagreementArraySizeException マトリクスと配列のサイズが一致しない場合
* @see jp.ac.takushoku_u.cs.SSAnalysis#getGraphElementArray()
*/
	public void setHeightArray(int[] heightArray){
		if(matrix.getRowLength() == heightArray.length){
			paintNumber = new int[heightArray.length];
			System.arraycopy(heightArray, 0, paintNumber, 0, heightArray.length);
		}
		else{
			throw new DisagreementArraySizeException("matrix("+matrix.getRowLength()+
			"),array("+heightArray.length+")");
		}
	}
	
/**
* 設定された縦軸情報からより上に描く順序を求めます。
* @param averageArray 縦軸情報
* @return 要素を上から描く順に項目番号を並び替えた配列。
*/
	private int[] getGraphArray(double[] averageArray){
		int[] returnArray = new int[matrix.getColLength()];		//平均評定値の高い順に並べた時の列番号（項目）を入れる
		
		for (int i = 0; i < averageArray.length; i++){
			double maxAverage = averageArray[0];		//初期値
			int maxNumber = 0;
			for (int j = 1; j <  averageArray.length; j++){		//比較を行う
				if (maxAverage < averageArray[j]){
					maxAverage = averageArray[j];
					maxNumber = j;
				}
			}
			averageArray[maxNumber] = Double.NEGATIVE_INFINITY;	 //比較対象からはずす
			returnArray[i] = maxNumber;
		}
		return returnArray;
	}
	
/**
* 最大横幅を探索します。
* @param levelArray レベルが入った配列
* @return 最大横幅
*/
	private int getColMaxLevel(int[] levelArray){
		int maxCount = 0;
		for(int i = 0; i < levelArray.length; i++){
			int count = 0;
			for(int j = 0; j < levelArray.length; j++){
				if(levelArray[j] == i){
					count++;
				}
			}
			if(count > maxCount){
				maxCount = count;
			}
		}
		
		return maxCount;
	}
	
/**
* 各レベルでの配置順を配列として返します。
* @return 配置順を格納した配列
*/
	private int[] getWidePointArray(){
		int[] levelArray = new int[matrix.getRowLength()];
		System.arraycopy(getPaintLevel(), 0, levelArray, 0, matrix.getRowLength());
		int maxWide = getRowMaxLevel(levelArray);
		int[] wide = new int[maxWide+1];
		for(int i = 0; i < wide.length; i++){
			wide[i] = 1;
		}
		
		int[] returnArray = new int[matrix.getRowLength()];
		for(int i = 0; i < matrix.getRowLength(); i++){
			returnArray[i] = wide[levelArray[i]];
			wide[levelArray[i]]++;
		}
		return returnArray;
	}
	
/**
* 最大レベルを探索します。
* @param levelArray レベルが入った配列
* @return 最大レベル
*/
	private int getRowMaxLevel(int[] levelArray){
		int num = 0;
		for(int i = 0; i < levelArray.length; i++){
			if(levelArray[i] > num){
				num = levelArray[i];
			}
		}
		return num;
	}
	
/**
* 指定したレベルの横幅を探索します。
* @param level 指定するレベル
* @param levelArray レベルが入った配列
* @return 横幅
*/
	private int getColLevel(int level, int[] levelArray){
		int count = 0;
		for(int i = 0; i < levelArray.length; i++){
			if(levelArray[i] == level){
				count++;
			}
		}
		
		return count;
	}
	
/**
* グラフを描く際の同一レベルでの位置番号を配列として返します。
* 値が小さいものほど、そのレベル内では左に描画を行います。
* @return 位置を格納した配列(配列の大きさは描画を行う項目数)
*/
	public int[] getPaintWideNumber(){
		paintWideNumber = new int[elementNumber];
		System.arraycopy(getWidePointArray(), 0, paintWideNumber, 0, elementNumber);
		return paintWideNumber;
	}
	
/**
* 各レベルの横幅情報を保持した配列を返します。
* @return 横幅情報の配列
*/
	public int[] getPaintWide(){
		paintWide = new int[getRowMaxLevel(getPaintLevel())+1];
			for(int i = 0; i <= getRowMaxLevel(getPaintLevel()); i++){
				paintWide[i] = getColLevel(i, getPaintLevel());
		}
		return paintWide;
	}
	
}